#define FMI_VERSION 3

#include <fmi3Functions.h>
#include <stdlib.h>
#include <string.h>
#include "monitor.c"

static const fmi3String monitor_uuid = "{286BDCB0-F8B0-11ED-93C1-ACDE48001122}";

struct monitor_fmu {
  char *instanceName;    /* const */  int current_loc;       /* in */
  monitor_input_t input; /* in */
  RV_value output;       /* out */
  int soft_reset;        /* in */
};

static int dimension_table[] = {1, 1, 0};

static fmi3String constants[] = {
  NULL
};

/* FMI functions */
/* Inquire version numbers and set debug logging */
const char* fmi3GetVersion () { return fmi3Version; }

fmi3Status fmi3SetDebugLogging (fmi3Instance _instance, fmi3Boolean loggingOn,
                                size_t nCategories,
                                const fmi3String categories[])
{ return fmi3Discard; }

/* Creation and destruction of FMU instances */
fmi3Instance fmi3InstantiateCoSimulation (
             fmi3String                     instanceName,
             fmi3String                     instantiationToken,
             fmi3String                     resourcePath,
             fmi3Boolean                    visible,
             fmi3Boolean                    loggingOn,
             fmi3Boolean                    eventModeUsed,
             fmi3Boolean                    earlyReturnAllowed,
             const fmi3ValueReference       requiredIntermediateVariables[],
             size_t                         nRequiredIntermediateVariables,
             fmi3InstanceEnvironment        instanceEnvironment,
             fmi3LogMessageCallback         logMessage,
             fmi3IntermediateUpdateCallback intermediateUpdate)
{
  struct monitor_fmu *_monitor;

  if (strcmp(instantiationToken, monitor_uuid) != 0) { return NULL; }
  _monitor = (struct monitor_fmu *)calloc(1, sizeof(struct monitor_fmu));
  _monitor->instanceName = strdup(instanceName);
  _monitor->current_loc = 1; /* initial location */
  return (fmi3Instance)_monitor;
}

fmi3Instance fmi3InstantiateModelExchange (
             fmi3String                 instanceName,
             fmi3String                 instantiationToken,
             fmi3String                 resourcePath,
             fmi3Boolean                visible,
             fmi3Boolean                loggingOn,
             fmi3InstanceEnvironment    instanceEnvironment,
             fmi3LogMessageCallback     logMessage)
{ return NULL; }

fmi3Instance fmi3InstantiateScheduledExecution (
             fmi3String                     instanceName,
             fmi3String                     instantiationToken,
             fmi3String                     resourcePath,
             fmi3Boolean                    visible,
             fmi3Boolean                    loggingOn,
             fmi3InstanceEnvironment        instanceEnvironment,
             fmi3LogMessageCallback         logMessage,
             fmi3ClockUpdateCallback        clockUpdate,
             fmi3LockPreemptionCallback     lockPreemption,
             fmi3UnlockPreemptionCallback   unlockPreemption)
{ return NULL; }

void fmi3FreeInstance (fmi3Instance _instance)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;

  free(_mon->instanceName);
  free(_mon);
}

/* Enter and exit initialization mode, terminate and reset */
fmi3Status fmi3EnterInitializationMode (fmi3Instance _instance,
                                  fmi3Boolean toleranceDefined,
                                         fmi3Float64 tolerance,
                                         fmi3Float64 startTime,
                                   fmi3Boolean stopTimeDefined,
                                          fmi3Float64 stopTime)
{ return fmi3OK; }

fmi3Status fmi3ExitInitializationMode (fmi3Instance _instance)
{ return fmi3OK; }

fmi3Status fmi3EnterEventMode (fmi3Instance _instance)
{ return fmi3OK; }

fmi3Status fmi3Terminate (fmi3Instance _instance) { return fmi3OK; }

fmi3Status fmi3Reset (fmi3Instance _instance)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  memset((char *)_mon, 0, sizeof(struct monitor_fmu));
  _mon->current_loc = 1;
  return fmi3OK;
}

fmi3Status fmi3GetFloat32 (fmi3Instance _instance,
                    const fmi3ValueReference vr[], size_t nvr,
                              fmi3Float32 value[], size_t nValues)
{ return fmi3Warning; }

fmi3Status fmi3GetFloat64 (fmi3Instance _instance,
                           const fmi3ValueReference vr[], size_t nvr,
                           fmi3Float64 value[], size_t nValues)
{ return fmi3Warning; }

fmi3Status fmi3GetBinary (fmi3Instance instance,
     const fmi3ValueReference valueReferences[],
                        size_t nValueReferences,
                            size_t valueSizes[],
                            fmi3Binary values[],
                                 size_t nValues)
{ return fmi3Warning; }

fmi3Status fmi3GetClock (fmi3Instance instance,
    const fmi3ValueReference valueReferences[],
                       size_t nValueReferences,
                            fmi3Clock values[])
{ return fmi3Warning; }

fmi3Status fmi3GetBoolean
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               fmi3Boolean value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];

    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    case 0: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.p;
      break;
    }
    case 1: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.q;
      break;
    }
    case 2:
      value[offset] = _mon->output;
      break;
    default:
      status = fmi3Warning;
    }
  }
  return status;
}

fmi3Status fmi3GetInt8
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               fmi3Int8 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];

    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    case 0: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.p;
      break;
    }
    case 1: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.q;
      break;
    }
    case 2:
      value[offset] = _mon->output;
      break;
    default:
      status = fmi3Warning;
    }
  }
  return status;
}

fmi3Status fmi3GetUInt8
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               fmi3UInt8 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];

    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    case 0: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.p;
      break;
    }
    case 1: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.q;
      break;
    }
    case 2:
      value[offset] = _mon->output;
      break;
    default:
      status = fmi3Warning;
    }
  }
  return status;
}

fmi3Status fmi3GetInt16
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               fmi3Int16 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];

    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    case 0: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.p;
      break;
    }
    case 1: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.q;
      break;
    }
    case 2:
      value[offset] = _mon->output;
      break;
    default:
      status = fmi3Warning;
    }
  }
  return status;
}

fmi3Status fmi3GetUInt16
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               fmi3UInt16 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];

    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    case 0: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.p;
      break;
    }
    case 1: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.q;
      break;
    }
    case 2:
      value[offset] = _mon->output;
      break;
    default:
      status = fmi3Warning;
    }
  }
  return status;
}

fmi3Status fmi3GetInt32
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               fmi3Int32 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];

    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    case 0: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.p;
      break;
    }
    case 1: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.q;
      break;
    }
    case 2:
      value[offset] = _mon->output;
      break;
    default:
      status = fmi3Warning;
    }
  }
  return status;
}

fmi3Status fmi3GetUInt32
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               fmi3UInt32 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];

    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    case 0: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.p;
      break;
    }
    case 1: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.q;
      break;
    }
    case 2:
      value[offset] = _mon->output;
      break;
    default:
      status = fmi3Warning;
    }
  }
  return status;
}

fmi3Status fmi3GetInt64
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               fmi3Int64 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];

    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    case 0: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.p;
      break;
    }
    case 1: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.q;
      break;
    }
    case 2:
      value[offset] = _mon->output;
      break;
    default:
      status = fmi3Warning;
    }
  }
  return status;
}

fmi3Status fmi3GetUInt64
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               fmi3UInt64 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];

    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    case 0: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.p;
      break;
    }
    case 1: {
      if (offset >= nValues) return fmi3Error;
      value[offset] = _mon->input.q;
      break;
    }
    case 2:
      value[offset] = _mon->output;
      break;
    default:
      status = fmi3Warning;
    }
  }
  return status;
}

fmi3Status fmi3GetString
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               fmi3String value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  char v_str_temp[32];
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];

    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    case 2:
      value[offset] = RV_value_string(_mon->output);
      break;
    default:
      status = fmi3Warning;
    }
  }
  return status;
}

fmi3Status fmi3SetBoolean
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               const fmi3Boolean value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];
    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    case 0: {
      if (offset >= nValues) return fmi3Error;
      _mon->input.p = value[offset];
      break;
    }
    case 1: {
      if (offset >= nValues) return fmi3Error;
      _mon->input.q = value[offset];
      break;
    }
    case 3:
      _mon->soft_reset = value[offset];
      break;
    default:
      status = fmi3Discard;
    }
  }
  return status;
}

fmi3Status fmi3SetInt8
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               const fmi3Int8 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];
    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    default:
      status = fmi3Discard;
    }
  }
  return status;
}

fmi3Status fmi3SetUInt8
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               const fmi3UInt8 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];
    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    default:
      status = fmi3Discard;
    }
  }
  return status;
}

fmi3Status fmi3SetInt16
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               const fmi3Int16 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];
    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    default:
      status = fmi3Discard;
    }
  }
  return status;
}

fmi3Status fmi3SetUInt16
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               const fmi3UInt16 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];
    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    default:
      status = fmi3Discard;
    }
  }
  return status;
}

fmi3Status fmi3SetInt32
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               const fmi3Int32 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];
    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    default:
      status = fmi3Discard;
    }
  }
  return status;
}

fmi3Status fmi3SetUInt32
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               const fmi3UInt32 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];
    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    default:
      status = fmi3Discard;
    }
  }
  return status;
}

fmi3Status fmi3SetInt64
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               const fmi3Int64 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];
    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    default:
      status = fmi3Discard;
    }
  }
  return status;
}

fmi3Status fmi3SetUInt64
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               const fmi3UInt64 value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];
    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    default:
      status = fmi3Discard;
    }
  }
  return status;
}

fmi3Status fmi3SetString
              (fmi3Instance _instance,
               const fmi3ValueReference vr[], size_t nvr,
               const fmi3String value[], size_t nValues)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  size_t i, j, offset;
  fmi3Status status = fmi3OK;

  if (nvr > 0 && NULL == vr)    { return fmi3Error; }
  if (nvr > 0 && NULL == value) { return fmi3Error; }
  if (nvr > nValues) { return fmi3Error; }

  for (i = 0; i < nvr; i++) {
    fmi3ValueReference r = vr[i];
    for (j = 0, offset = 0; j < i; j++) {
      offset += dimension_table[vr[j]];
    }
    switch (r) {
    default:
      status = fmi3Discard;
    }
  }
  return status;
}

fmi3Status fmi3SetFloat32 (fmi3Instance _instance,
                           const fmi3ValueReference vr[], size_t nvr,
                           const fmi3Float32 value[], size_t nValues)
{ return fmi3Discard; }

fmi3Status fmi3SetFloat64 (fmi3Instance _instance,
                           const fmi3ValueReference vr[], size_t nvr,
                           const fmi3Float64 value[], size_t nValues)
{ return fmi3Discard; }

fmi3Status fmi3SetBinary (fmi3Instance instance,
                          const fmi3ValueReference valueReferences[],
                          size_t nValueReferences,
                          const size_t valueSizes[],
                          const fmi3Binary values[],
                          size_t nValues)
{ return fmi3Discard; }

fmi3Status fmi3SetClock (fmi3Instance instance,
                         const fmi3ValueReference valueReferences[],
                         size_t nValueReferences,
                         const fmi3Clock values[])
{ return fmi3Discard; }

fmi3Status fmi3GetNumberOfVariableDependencies (fmi3Instance instance,
                                    fmi3ValueReference valueReference,
                                                size_t* nDependencies)
{ return fmi3Warning; }
fmi3Status fmi3GetVariableDependencies (fmi3Instance instance,
                                  fmi3ValueReference dependent,
                            size_t elementIndicesOfDependent[],
                             fmi3ValueReference independents[],
                         size_t elementIndicesOfIndependents[],
                          fmi3DependencyKind dependencyKinds[],
                                          size_t nDependencies)
{ return fmi3Warning; }
fmi3Status fmi3GetDirectionalDerivative (fmi3Instance instance,
                           const fmi3ValueReference unknowns[],
                                              size_t nUnknowns,
                             const fmi3ValueReference knowns[],
                                                size_t nKnowns,
                                      const fmi3Float64 seed[],
                                                  size_t nSeed,
                                     fmi3Float64 sensitivity[],
                                           size_t nSensitivity)
{ return fmi3Warning; }
fmi3Status fmi3GetAdjointDerivative (fmi3Instance instance,
                       const fmi3ValueReference unknowns[],
                                          size_t nUnknowns,
                         const fmi3ValueReference knowns[],
                                            size_t nKnowns,
                                  const fmi3Float64 seed[],
                                              size_t nSeed,
                                 fmi3Float64 sensitivity[],
                                       size_t nSensitivity)
{ return fmi3Warning; }
fmi3Status fmi3EnterConfigurationMode (fmi3Instance instance)
{ return fmi3OK; }
fmi3Status fmi3ExitConfigurationMode (fmi3Instance instance)
{ return fmi3OK; }
fmi3Status fmi3GetFMUState (fmi3Instance _instance, fmi3FMUState *FMUState)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  int *current_loc;
  if (FMUState == NULL) { return fmi3Error; }
  if (*FMUState == NULL) {
    current_loc = (int *)malloc(sizeof(int));
    *FMUState = (fmi3FMUState)current_loc;
  } else {
    current_loc = (int *)(*FMUState);
  }
  *current_loc = _mon->current_loc;
  return fmi3OK;
}

fmi3Status fmi3SetFMUState (fmi3Instance _instance, fmi3FMUState FMUState)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  int *current_loc;
  if (FMUState == NULL) { return fmi3Error; }
  current_loc = (int *)FMUState;
  _mon->current_loc = *current_loc;
  return fmi3OK;
}

fmi3Status fmi3FreeFMUState (fmi3Instance _instance, fmi3FMUState *FMUState)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  int *current_loc;
  if (FMUState == NULL) { return fmi3Error; }
  current_loc = (int *)(*FMUState);
  *FMUState = NULL;
  if (current_loc == NULL) { return fmi3OK; }
  free(current_loc);
  return fmi3OK;
}

fmi3Status fmi3SerializedFMUStateSize (fmi3Instance _instance,
                                       fmi3FMUState FMUState, size_t *size)
{ return fmi3Warning; }

fmi3Status fmi3SerializeFMUState (fmi3Instance _instance,
                                  fmi3FMUState FMUState,
                                  fmi3Byte serializedState[], size_t size)
{ return fmi3Warning; }

fmi3Status fmi3DeserializeFMUState (fmi3Instance _instance,
                                    const fmi3Byte serializedState[],
                                    size_t size, fmi3FMUState* FMUState)
{ return fmi3Warning; }

/* Simulating the FMU */
fmi3Status fmi3EnterStepMode (fmi3Instance _instance)
{ return fmi3OK; }

fmi3Status fmi3GetOutputDerivatives (fmi3Instance _instance,
                                     const fmi3ValueReference vr[], size_t nvr,
                                     const fmi3Int32 orders[],
                                     fmi3Float64 values[], size_t nValues)
{ return fmi3Warning; }

fmi3Status fmi3DoStep (fmi3Instance _instance,
                       fmi3Float64 currentCommunicationPoint,
                       fmi3Float64 communicationStepSize,
                       fmi3Boolean noSetFMUStatePriorToCurrentPoint,
                       fmi3Boolean* eventHandlingNeeded,
                       fmi3Boolean* terminateSimulation,
                       fmi3Boolean* earlyReturn,
                       fmi3Float64* lastSuccessfulTime)
{
  struct monitor_fmu *_mon = (struct monitor_fmu *)_instance;
  monitor_input_t *input = &(_mon->input);
  int *current_loc = &(_mon->current_loc);
  monitor_mask_t *mask = NULL;
  RV_reset reset = NO_RESET;
  RV_value output;

  if (_mon->soft_reset) {
    reset = SOFT_RESET;
    _mon->soft_reset = 0;
  }
  output = monitor_scalar(input, mask, reset, current_loc);
  /* FMI 3.0: advance the clock */
  *lastSuccessfulTime = currentCommunicationPoint + communicationStepSize;
  *eventHandlingNeeded = false;
  *terminateSimulation = false;
  *earlyReturn = false;
  /* final steps */
  if (output <= RV_ERROR) {
    _mon->output = output;
    return fmi3OK;
  } else {
    return fmi3Error;
  }
}

fmi3Status fmi3GetIntervalDecimal (fmi3Instance instance,
              const fmi3ValueReference valueReferences[],
                                 size_t nValueReferences,
                                 fmi3Float64 intervals[],
                      fmi3IntervalQualifier qualifiers[])
{ return fmi3Warning; }

fmi3Status fmi3GetIntervalFraction (fmi3Instance instance,
               const fmi3ValueReference valueReferences[],
                                  size_t nValueReferences,
                                    fmi3UInt64 counters[],
                                 fmi3UInt64 resolutions[],
                       fmi3IntervalQualifier qualifiers[])
{ return fmi3Warning; }

fmi3Status fmi3GetShiftDecimal (fmi3Instance instance,
           const fmi3ValueReference valueReferences[],
                              size_t nValueReferences,
                                 fmi3Float64 shifts[])
{ return fmi3Warning; }

fmi3Status fmi3GetShiftFraction (fmi3Instance instance,
            const fmi3ValueReference valueReferences[],
                               size_t nValueReferences,
                                 fmi3UInt64 counters[],
                              fmi3UInt64 resolutions[])
{ return fmi3Warning; }

fmi3Status fmi3SetIntervalDecimal (fmi3Instance instance,
              const fmi3ValueReference valueReferences[],
                                 size_t nValueReferences,
                           const fmi3Float64 intervals[])
{ return fmi3Discard; }

fmi3Status fmi3SetIntervalFraction (fmi3Instance instance,
               const fmi3ValueReference valueReferences[],
                                  size_t nValueReferences,
                              const fmi3UInt64 counters[],
                           const fmi3UInt64 resolutions[])
{ return fmi3Discard; }

fmi3Status fmi3SetShiftDecimal (fmi3Instance instance,
           const fmi3ValueReference valueReferences[],
                              size_t nValueReferences,
                           const fmi3Float64 shifts[])
{ return fmi3Discard; }

fmi3Status fmi3SetShiftFraction (fmi3Instance instance,
            const fmi3ValueReference valueReferences[],
                               size_t nValueReferences,
                           const fmi3UInt64 counters[],
                        const fmi3UInt64 resolutions[])
{ return fmi3Discard; }

fmi3Status fmi3EvaluateDiscreteStates (fmi3Instance instance)
{ return fmi3OK; }

fmi3Status fmi3UpdateDiscreteStates (fmi3Instance instance,
                     fmi3Boolean* discreteStatesNeedUpdate,
                          fmi3Boolean* terminateSimulation,
            fmi3Boolean* nominalsOfContinuousStatesChanged,
              fmi3Boolean* valuesOfContinuousStatesChanged,
                         fmi3Boolean* nextEventTimeDefined,
                                fmi3Float64* nextEventTime)
{ return fmi3Warning; }

fmi3Status fmi3EnterContinuousTimeMode (fmi3Instance instance)
{ return fmi3OK; }

fmi3Status fmi3CompletedIntegratorStep (fmi3Instance instance,
                fmi3Boolean  noSetFMUStatePriorToCurrentPoint,
                                  fmi3Boolean* enterEventMode,
                             fmi3Boolean* terminateSimulation)
{ return fmi3Warning; }

fmi3Status fmi3SetTime (fmi3Instance instance, fmi3Float64 time)
{ return fmi3Discard; }

fmi3Status fmi3GetEventIndicators (fmi3Instance instance,
                           fmi3Float64 eventIndicators[],
                                 size_t nEventIndicators)
{ return fmi3Warning; }

fmi3Status fmi3GetContinuousStates (fmi3Instance instance,
                           fmi3Float64 continuousStates[],
                                 size_t nContinuousStates)
{ return fmi3Warning; }

fmi3Status fmi3SetContinuousStates (fmi3Instance instance,
                     const fmi3Float64 continuousStates[],
                                 size_t nContinuousStates)
{ return fmi3Error; }

fmi3Status fmi3GetContinuousStateDerivatives (fmi3Instance instance,
                                          fmi3Float64 derivatives[],
                                           size_t nContinuousStates)
{ return fmi3Error; }

fmi3Status fmi3GetNominalsOfContinuousStates (fmi3Instance instance,
                                             fmi3Float64 nominals[],
                                           size_t nContinuousStates)
{ return fmi3Error; }

fmi3Status fmi3GetNumberOfEventIndicators (fmi3Instance instance,
                                        size_t* nEventIndicators)
{ return fmi3Error; }

fmi3Status fmi3GetNumberOfContinuousStates (fmi3Instance instance,
                                        size_t* nContinuousStates)
{ return fmi3Error; }

fmi3Status fmi3ActivateModelPartition (fmi3Instance instance,
                           fmi3ValueReference clockReference,
                                  fmi3Float64 activationTime)
{ return fmi3Error; }

