/* ---------------------------------------------------------------------------

  This file is part of NuRV (code generation sample code)
  Copyright (c) 2018-2021 Fondazione Bruno Kessler (FBK), Italy

  Some code pieces were learnt from Kaspersky Security System (KSS) Demos

  Author: Chun Tian <ctian@fbk.eu>
-----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>

#include "monitor.h"

static int current_loc = 1;
static int reset_p = 0;

/**
 * Print current time to standard output.
 */
static void print_time()
{
    char buf[20];
    size_t buf_len;
    struct timeval now;
    int milli;

    gettimeofday(&now, NULL);
    milli = now.tv_usec / 1000;
    buf_len = strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S", localtime(&now.tv_sec));
    if (buf_len > 0)
        printf("%s.%03d ", buf, milli);
}

static int heartbeat (int p_or_q)
{
  monitor_input_t input; /* scalar API */
  int output;
  int retval;
  // long state;

  if (p_or_q) {
    input.p = 1;
    input.q = 0;
    // state = 1L; /* !q & p */
  } else {
    input.p = 0;
    input.q = 1;
    // state = 2L; /* q & !p */
  }

  output = monitor_scalar (&input, NULL, reset_p, &current_loc);
  reset_p = 0;

  print_time();
  printf("[Command] HEARTBEAT(%s) ... ", (p_or_q ? "p" : "q"));

  switch (output) {
  case RV_TRUE:
    printf("SUCCESS\n");
    retval = 1; /* reset then */
    break;
  case RV_FALSE:
    printf("FAIL\n");
    retval = 1; /* reset then */
    break;
  case RV_UNKNOWN:
    printf("OK\n");
    retval = 0; /* continue the loop */
    break;
  default:
    printf("ERROR\n");
    exit(-1);
  }

  return retval;
}

static int reset ()
{
  /* hard reset the monitor */
  reset_p = 1;

  print_time();
  printf("[Command] RESET\n"); /* one way function */

  return 0;
}

/*
 * main 
 */
int main (int argc, char* argv[])
{
  int i, p_or_q;

  srand(time(NULL));

  for (i = 0; i < 16; i++) {
    p_or_q = rand() % 2;
    if (heartbeat(p_or_q) != 0) {
      (void) reset();
    }
    usleep(200000);
  }
  printf("\n");

  exit (0);
}
