/******************************************************************************
 * monitor code generated by NuRV (version: 1.8.0)
 *
 * LTL:  G (p ->  X ( X q))
 *
 * nbits_obs: 2 (width: 1), nbits_all: 4 (width: 2)
 * reset_table: 1, trans_table: 1, input_filter: 1
 ******************************************************************************/

package eu.fbk.rvsynth;

import java.util.Arrays;
import java.util.Comparator;

public class Monitor extends BaseMonitor {
  private int current_loc = 1;

  // old API
  public int // 0 = unknown, 1 = true, 2 = false, 3 = error
    run (long state,
         int reset) // 0 = none, 1 = hard, 2 = soft
  {
    long[] states = new long[2];
    long base = 1L;
    RV_value output;

    states[0] = (state % 2L) * base;
    state >>= 1;
    base <<= 1;
    states[0] += (state % 2L) * base;
    state >>= 1;
    base = 1L; /* next segment */
    states[1] = (state % 2L) * base;
    state >>= 1;
    base <<= 1;
    states[1] += (state % 2L) * base;

    output = run(states, RV_reset.valueOfLabel(reset));
    return output.label;
  }

  // new API
  public RV_value run(long[] states, RV_reset reset)
  {
    // local variables
    long[] inputs = new long[1];
    long[][] table;
    int result;
    long[] key = new long[3];
    int size;
    RV_value output;

    // handle hard resets
    if (reset.equals(RV_reset.HARD_RESET)) {
      current_loc = 1; // initial location
    }
    else if (reset.equals(RV_reset.SOFT_RESET)) {
      int i = current_loc;
      current_loc = reset_table[i];
    }

    // validate inputs
    if (current_loc <= 0 || current_loc > 7) {
      return RV_value.RV_INVALID_LOC;
    }
    if (states.length < 2) {
      return RV_value.RV_INVALID_ARG;
    }

    // input filtering
    {
      long state = states[0], base = 1L;
      inputs[0] = (state % 2L) * base;
      state >>= 1; base <<= 1;
      inputs[0] += (state % 2L) * base;
    }

    // perform binary search
    table = data_table[current_loc];
    size = size_table[current_loc];
    key[0] = inputs[0];
    result = Arrays.binarySearch(table, 0, size - 1, key, new data_cmp());
    if (0 <= result && result < size) {
      long[] obj = table[result];
      output = RV_value.valueOfLabel((int)obj[1]); // output
      current_loc = (int)obj[2]; // next_loc
    } else {
      output = RV_value.RV_ERROR;
    }
    return output;
  }

  // maximum number of elements in each line: 20
  private static final int reset_table[] = { // size: 8
    0, 1, 1, 1, 1, 1, 1, 1
  };

  // maximum number of elements in each line: 4
  private static final long[][] data_7 = { // size: 2
    {0x1L, 2L, 6L}, {0x2L, 2L, 7L}
  };

  private static final long[][] data_6 = { // size: 2
    {0x1L, 2L, 6L}, {0x2L, 2L, 7L}
  };

  private static final long[][] data_5 = { // size: 2
    {0x1L, 2L, 6L}, {0x2L, 0L, 3L}
  };

  private static final long[][] data_4 = { // size: 2
    {0x1L, 2L, 6L}, {0x2L, 0L, 5L}
  };

  private static final long[][] data_3 = { // size: 2
    {0x1L, 0L, 2L}, {0x2L, 0L, 3L}
  };

  private static final long[][] data_2 = { // size: 2
    {0x1L, 0L, 4L}, {0x2L, 0L, 5L}
  };

  private static final long[][] data_1 = { // size: 2
    {0x1L, 0L, 2L}, {0x2L, 0L, 3L}
  };

  private static final long[][][] data_table = { // size: 8
    null, data_1, data_2, data_3, data_4, data_5, data_6, data_7
  };

  // number of elements in each line: 20
  private static final int size_table[] = { // size: 8
    0, 2, 2, 2, 2, 2, 2, 2
  };

  private class data_cmp implements Comparator<long []> {
    public int compare(long[] o1, long[] o2) {
      if (o1[0] < o2[0]) {
        return -1;
      } else if (o1[0] > o2[0]) {
        return 1;
      } else {
        return 0;
      }
    }
  }
} // end of class
