(in-package :rvsynth)

;; reset table
(defparameter *monitor-reset-table*
  #(0 1 1 1 1 1 1 1))

(defparameter *monitor-data-7+
  '((#x1 2 6) (#x2 2 7)))

(defparameter *monitor-data-6+
  '((#x1 2 6) (#x2 2 7)))

(defparameter *monitor-data-5+
  '((#x1 2 6) (#x2 0 3)))

(defparameter *monitor-data-4+
  '((#x1 2 6) (#x2 0 5)))

(defparameter *monitor-data-3+
  '((#x1 0 2) (#x2 0 3)))

(defparameter *monitor-data-2+
  '((#x1 0 4) (#x2 0 5)))

(defparameter *monitor-data-1+
  '((#x1 0 2) (#x2 0 3)))

(defparameter *monitor-data-table*
  '#(nil
     *monitor-data-1+ *monitor-data-2+ *monitor-data-3+ *monitor-data-4+ ; L1
     *monitor-data-5+ *monitor-data-6+ *monitor-data-7+))

(defparameter *monitor-size-table*
  #(0 2 2 2 2 2 2 2))

;;; base monitor class
(defclass base-monitor ()
  ((current-loc :type fixnum :accessor current-loc :initform 1)
   (reset-table :type simple-vector :reader reset-table)
   (trans-table :type simple-vector :reader trans-table))
  (:documentation "NuRV base monitor class"))

;;; generic function
(defgeneric run (instance states reset)
  (:documentation "monitor entry function"))

;; per-monitor structure
(defclass monitor-trans
  ((input    :reader monitor-input  :initarg :input)
   (output   :reader monitor-output :initarg :output)
   (next-loc :reader next-loc :initarg :next-loc))
  (:documentation "per-monitor transition table entry"))

;;; monitor class
(defclass monitor (base-monitor)
  ()
  (:documentation "NuRV monitor class"))

(defmethod initialize-instance :after ((instance monitor) &rest initargs)
  (declare (ignore initargs))
  (with-slots (reset-table trans-table) instance
    (setq reset-table (copy-seq *monitor-reset-table*))
    (setq trans-table (make-sequence 'simple-vector 8
                                     :initial-element nil))
    (loop for i from 1 upto 7
          as h = (make-hash-table :test 'equal :size (svref *monitor-size-table* i))
          as l = (symbol-value (svref *monitor-data-table* i)) do
      (setf (svref trans-table i) h)
      (loop for (input output next-loc) in l do
        (setf (gethash (list input) h)
              (make-instance 'monitor-trans
                :input input :output output :next-loc next-loc))))))

;;; old API
(defmethod run ((instance monitor) (state fixnum) (reset symbol))
  (let ((states (make-sequence 'simple-vector 1 :initial-element 0))
        (base 1))
    (incf (svref states 0) (* base (mod state 2)))
    (setq state (floor state 2))
    (setq base (* base 2))
    (incf (svref states 0) (* base (mod state 2)))
    (run instance states reset)))

;;; new API
(defmethod run ((instance monitor) (states sequence) (reset symbol))
  ;; validate inputs
  (if (not (and (<= 1 (current-loc instance))
                (<= (current-loc instance) 7)))
      (return-from run :error))
  ;; handle resets
  (case reset
    (:hard (setf (current-loc instance) 1))
    (:soft (setf (current-loc instance)
                 (svref (reset-table instance) (current-loc instance)))))
  ;; input filtering (if necessary)
  (let ((inputs states))
    ;; perform hash-table search
    (let* ((table (svref (trans-table instance) (current-loc instance)))
           (result (gethash (concatenate 'list inputs) table)))
      (cond (result
             (setf (current-loc instance) (monitor-next-loc result))
             (ecase (monitor-output result)
               (1 :true)
               (2 :false)
               (0 :unknown)))
            (t :error)))))
