/* ---------------------------------------------------------------------------

  This file is part of the ``rvsynth'' package (client side).
  Copyright (R) 2018  Fondazione Bruno Kessler

  Some code pieces were from Kaspersky Security System Demos

  Author: Chun Tian
-----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include <string.h>

#include "monitor.h"

static int current_loc = 1;

/**
 * Print current time to standard output.
 */
static void print_time()
{
    char buf[20];
    size_t buf_len;
    struct timeval now;
    int milli;

    gettimeofday(&now, NULL);
    milli = now.tv_usec / 1000;
    buf_len = strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S", localtime(&now.tv_sec));
    if (buf_len > 0)
        printf("%s.%03d ", buf, milli);
}

static int heartbeat (int is_missed)
{
  RV_value output;
  int retval;
  monitor_input_t state;

  if (is_missed) {
    state.beatMissed = 1;
    state.beatOk = 0;
  } else {
    state.beatMissed = 0;
    state.beatOk = 1;
  }

  output = monitor_scalar(&state, NULL, 0, &current_loc);

  print_time();
  printf("[Command] HEARTBEAT(%d) ... ", is_missed);

  switch (output) {
  case RV_TRUE:
    printf("SUCCESS\n");
    retval = 1; /* reset then */
    break;
  case RV_FALSE:
    printf("FAIL\n");
    retval = 1; /* reset then */
    break;
  case RV_UNKNOWN:
    printf("OK\n");
    retval = 0; /* continue the loop */
    break;
  default:
    printf("ERROR\n");
    exit(-1);
  }

  return retval;
}

static int reset ()
{
  /* reset (stop) the monitor */
  current_loc = 1;

  print_time();
  printf("[Command] RESET\n"); /* one way function */

  return 0;
}

/*
 * main 
 */
int main (int argc, char* argv[])
{
  int i, is_missed;

  srand(time(NULL));

  for (i = 0; i < 16; i++) {
    is_missed = rand() % 2;
    if (heartbeat(is_missed) != 0) {
      (void) reset();
    }
    usleep(200000);
  }
  printf("\n");

  exit (0);
}
