#include <stdio.h>
#include <stdlib.h>

typedef enum {
  RV_UNKNOWN     = 0,
  RV_TRUE        = 1,
  RV_FALSE       = 2,
  RV_ERROR       = 3,
  RV_INVALID_ARG = 4,
  RV_INVALID_LOC = 5,
  RV_INVALID_MON = 6,
} RV_value;

#ifdef __GNUC__
#define RV_value_string(z) \
     ({__typeof__(z) x = z; \
       (x == RV_UNKNOWN     ? "unknown" : \
        x == RV_TRUE        ? "true" : \
        x == RV_FALSE       ? "false" : \
        x == RV_ERROR       ? "out-of-model" : \
        x == RV_INVALID_ARG ? "invalid inputs" : \
        x == RV_INVALID_LOC ? "invalid location" : "");})
#else
#define RV_value_string(x) \
       (x == RV_UNKNOWN     ? "unknown" : \
        x == RV_TRUE        ? "true" : \
        x == RV_FALSE       ? "false" : \
        x == RV_ERROR       ? "out-of-model" : \
        x == RV_INVALID_ARG ? "invalid inputs" : \
        x == RV_INVALID_LOC ? "invalid location" : "")
#endif

// BEGIN
#ifdef __cplusplus
typedef bool boolean;
#else
typedef enum {false=0, true=1} boolean;
#endif

#ifdef __cplusplus
extern "C" {
#endif
typedef void*         BddEnc_ptr;
typedef void*          DDMgr_ptr;
typedef void*       ErrorMgr_ptr;
typedef void*       NodeList_ptr;
typedef void*      EnvObject_ptr;
typedef void*       NuSMVEnv_ptr;
typedef void*      StreamMgr_ptr;
typedef void*    OptsHandler_ptr;
typedef void*      SymbTable_ptr;
typedef void*           Prop_ptr;
typedef void* PropDbAccessor_ptr;
typedef void*     RV_monitor_ptr;
typedef void*            bdd_ptr;
typedef void*           node_ptr;
typedef node_ptr        Expr_ptr;
typedef void*     cmp_struct_ptr;
typedef void*             lsList;
typedef void*           lsHandle;
typedef void*          lsGeneric;
typedef int             lsStatus;
typedef void (*FP_V_E)(NuSMVEnv_ptr);
typedef enum { TRANS_TYPE_INVALID = -1 } TransType;
typedef enum {
  Prop_Prop_Type_First = 100, /* Do not touch this */
  Prop_NoType,
  Prop_Ctl,
  Prop_Ltl
} Prop_Type;
#define Nil                    ((node_ptr)0)
#define BDD_ENC(self)          ((BddEnc_ptr) self)
#define DD_MGR(self)           ((DDMgr_ptr) self)
#define ERROR_MGR(self)        ((ErrorMgr_ptr) self)
#define NIL(type)              ((type *) 0)
#define OPTS_HANDLER(self)     ((OptsHandler_ptr) self)
#define PROP_DB_ACCESSOR(self) ((PropDbAccessor_ptr) self)
#define STREAM_MGR(self)       ((StreamMgr_ptr) self)
#define SYMB_TABLE(x)          ((SymbTable_ptr) x)
#define ENV_OBJECT(self)       ((EnvObject_ptr) self)
#define ENV_BDD_ENCODER        "+:_bdd_enc"
#define ENV_DD_MGR             "+2_dd_manager"
#define ENV_ERROR_MANAGER      "+&_error_manager"
#define ENV_OPTS_HANDLER       "+,_opts_handler"
#define ENV_PROP_DBA           "+/_prop_dba"
#define ENV_STREAM_MANAGER     "+\"_stream_mgr"
#define ENV_SYMB_TABLE         "+-_symb_table"
NuSMVEnv_ptr NuSMVEnv_create(void);
void NuSMVEnv_destroy(NuSMVEnv_ptr);
void* NuSMVEnv_get_value(NuSMVEnv_ptr, char*);
NuSMVEnv_ptr EnvObject_get_environment(EnvObject_ptr self);
void NuRV_Init(NuSMVEnv_ptr);
void NuRV_Quit(NuSMVEnv_ptr);
void NuSMVCore_init_data(void);
void NuSMVCore_init(NuSMVEnv_ptr, FP_V_E [][2], int);
void NuSMVCore_quit(NuSMVEnv_ptr);
void set_verbose_level(OptsHandler_ptr, int);
void bdd_free(DDMgr_ptr, bdd_ptr);
char *get_input_file(OptsHandler_ptr);
int cmp_struct_get_read_model(cmp_struct_ptr);
int cmp_struct_get_flatten_hrc(cmp_struct_ptr);
int cmp_struct_get_encode_variables(cmp_struct_ptr);
int cmp_struct_get_build_flat_model(cmp_struct_ptr);
int cmp_struct_get_build_bool_model(cmp_struct_ptr);
int cmp_struct_get_bmc_setup(cmp_struct_ptr);
int cmp_struct_get_build_model(cmp_struct_ptr);
int Parser_read_model(NuSMVEnv_ptr, char*);
int Parser_ReadIdentifierExprFromString(NuSMVEnv_ptr, const char*, node_ptr*);
int CompileFlatten_flatten_smv(NuSMVEnv_ptr, boolean);
int Compile_create_flat_model(NuSMVEnv_ptr);
int Compile_encode_variables(NuSMVEnv_ptr, char*, boolean);
int Compile_build_model(NuSMVEnv_ptr, TransType);
Prop_ptr PropDbAccessor_get_prop_at_index(PropDbAccessor_ptr, int);
int PropDbAccessor_prop_parse_and_add(PropDbAccessor_ptr, SymbTable_ptr, char*, Prop_Type, node_ptr);
lsList lsSingleton(lsGeneric, lsHandle*);
NodeList_ptr NodeList_create(void);
RV_monitor_ptr RV_build_monitor_from_props(NuSMVEnv_ptr, lsList, boolean, bdd_ptr, NodeList_ptr);
int RV_heartbeat_internal(RV_monitor_ptr, bdd_ptr);
int RV_reset_internal(RV_monitor_ptr, boolean);
RV_value RV_output_value(RV_monitor_ptr);
bdd_ptr simulate_get_constraints_from_string(NuSMVEnv_ptr, const char*, BddEnc_ptr, boolean, boolean);
extern cmp_struct_ptr cmps;
#ifdef __cplusplus
} /* extern "C" */
#endif
// END

int main(int argc, char **argv)
{
  int status;
  static char *input_file = "model1.smv";
  static char *property = "(i < 150 U i >= 150)";
  FP_V_E iq_fns[][2] = {{NuRV_Init, NuRV_Quit}};
  NuSMVEnv_ptr env = NuSMVEnv_create();
  NuSMVCore_init_data(); /* core_data (global variable) */
  NuSMVCore_init(env, iq_fns, sizeof(iq_fns)/sizeof(iq_fns[0]));

  StreamMgr_ptr streams =
    STREAM_MGR(NuSMVEnv_get_value(env, ENV_STREAM_MANAGER));
  ErrorMgr_ptr errmgr =
    ERROR_MGR(NuSMVEnv_get_value(env, ENV_ERROR_MANAGER));
  OptsHandler_ptr opts =
    OPTS_HANDLER(NuSMVEnv_get_value(env, ENV_OPTS_HANDLER));

  /* for debugging purposes */
  set_verbose_level(opts, 0);

  /* read the model from file */
  if (cmp_struct_get_read_model(cmps) == 0) {
    if (Parser_read_model(env, input_file)) return 1;
  }
  if (cmp_struct_get_flatten_hrc(cmps) == 0) {
    if (CompileFlatten_flatten_smv(env, true)) return 1;
  }
  if (cmp_struct_get_build_flat_model(cmps) == 0) {
    if (Compile_create_flat_model(env)) return 1;
  }
  if (cmp_struct_get_encode_variables(cmps) == 0) {
    if (Compile_encode_variables(env, NIL(char), false)) return 1;
  }
  if (cmp_struct_get_build_model(cmps) == 0) {
    if (Compile_build_model(env, TRANS_TYPE_INVALID)) return 1;
  }

  /* Build the monitor */
  const PropDbAccessor_ptr prop_dba =
    PROP_DB_ACCESSOR(NuSMVEnv_get_value(env, ENV_PROP_DBA));
  const SymbTable_ptr symb_table = SYMB_TABLE(NuSMVEnv_get_value(env, ENV_SYMB_TABLE));
  int propno = PropDbAccessor_prop_parse_and_add(prop_dba, symb_table, property,
                                                 Prop_Ltl, Nil);
  Prop_ptr prop = PropDbAccessor_get_prop_at_index(prop_dba, propno);
  lsList props = lsSingleton ((lsGeneric)prop, (lsHandle *)NULL);
  NodeList_ptr observables = NodeList_create();
  bdd_ptr bdd_constraints = (bdd_ptr) NULL;
  boolean ptLTL = false;
  RV_monitor_ptr monitor =
    RV_build_monitor_from_props(env, props, ptLTL, bdd_constraints, observables);

  /* Send inputs to the monitor */
  BddEnc_ptr bdd_enc = BDD_ENC(NuSMVEnv_get_value(env, ENV_BDD_ENCODER));
  DDMgr_ptr dd = DD_MGR(NuSMVEnv_get_value(env, ENV_DD_MGR));
  bdd_constraints =
    simulate_get_constraints_from_string(env, "i = 145", bdd_enc,
                                         false, /* allow_nexts */
                                         true   /* allow_inputs */);

  RV_heartbeat_internal (monitor, bdd_constraints);
  printf("%s\n", RV_value_string(RV_output_value(monitor)));

  RV_heartbeat_internal (monitor, bdd_constraints);
  printf("%s\n", RV_value_string(RV_output_value(monitor)));

  bdd_free(dd, bdd_constraints);
  bdd_constraints =
    simulate_get_constraints_from_string(env, "i = 155", bdd_enc,
                                         false, /* allow_nexts */
                                         true   /* allow_inputs */);

  RV_heartbeat_internal (monitor, bdd_constraints);
  printf("%s\n", RV_value_string(RV_output_value(monitor)));

  bdd_free(dd, bdd_constraints);
  NuSMVCore_quit(env);
  NuSMVEnv_destroy(env);
  return 0;
}
