#!/usr/bin/env python3

from enum import Enum
from ctypes import *
import pathlib

class RV_value(Enum):
    RV_UNKNOWN     = 0
    RV_TRUE        = 1
    RV_FALSE       = 2
    RV_ERROR       = 3
    RV_INVALID_ARG = 4
    RV_INVALID_LOC = 5

class RV_reset(Enum):
    NO_RESET       = 0
    HARD_RESET     = 1
    SOFT_RESET     = 2

# constants used in the model
b = 2
a = 3
c = 4

k_map = {
  0 : 0x1, 1 : 0x3, b : 0x2, a : 0x0
}

l_map = {
  b : 0x0, c : 0x1
}

class Monitor:
    def __init__(self):
        self.current_loc = 1
        # Load the shared library into ctypes
        libname = pathlib.Path().absolute() / "libm1.so"
        self.c_lib = CDLL(libname)
        self.c_lib.m1_ffi.restype = c_int
        self.c_lib.m1_ffi.argtypes = [c_long, c_int, c_int]
    
    # type: dict * RV_reset -> RV_value
    def run(self, input, reset):
        i = input['i']
        k = input['k']
        l = input['l']
        input0 = 0
        input0 += 1 if i < 150 else 0
        input0 += (1 if i >= 150 else 0) * 0x2 # 2^1
        bits = k_map[k]
        input0 += (1 if (bits >> 1) else 0) * 0x4
        input0 += (1 if (bits >> 0) else 0) * 0x8
        bits = l_map[l]
        input0 += (1 if (bits >> 0) else 0) * 0x10
        results = self.c_lib.m1_ffi(input0, 1, reset.value, self.current_loc)
        verdict = RV_value(results & 0b11)
        next_loc = results >> 2
        self.current_loc = next_loc
        return verdict

